import os.path


from depscan.lib.csaf import (add_vulnerabilities, cleanup_dict, cleanup_list,
                              format_references, get_acknowledgements,
                              get_products, get_ref_summary, import_csaf_toml,
                              import_root_component, parse_cvss, parse_cwe,
                              parse_revision_history, parse_toml,
                              verify_components_present, )


def test_parse_revision_history():
    # add revision entry w/existing entries when final
    tracking = {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "status": "final",
        "version": "2",
        "revision_history": [
            {
                "date": "2023-10-02T23:50:07",
                "number": "2",
                "summary": "Update",
            },
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            },
        ],
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "revision_history": [
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            },
            {
                "date": "2023-10-02T23:50:07",
                "number": "2",
                "summary": "Update",
            },
            {"date": "2023-10-03T00:21:34", "number": "3", "summary": "Update"},
        ],
        "status": "final",
        "version": "3",
    }

    # add revision entry w/no existing entries when final
    tracking = {
        "current_release_date": "2022-09-22T20:54:06.186927",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06.186927",
        "status": "final",
        "version": "",
        "revision_history": [],
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2022-09-22T20:54:06",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "revision_history": [
            {"date": "2022-09-22T20:54:06", "number": "1", "summary": "Initial"}
        ],
        "status": "final",
        "version": "1",
    }

    # do not add when status is not final
    tracking = {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "status": "draft",
        "version": "2",
        "revision_history": [
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            }
        ],
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "status": "draft",
        "version": "2",
        "revision_history": [
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            }
        ],
    }

    # deal with a revision history inconsistent with the version number
    tracking = {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "status": "final",
        "version": "5",
        "revision_history": [
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            }
        ],
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2023-10-03T00:21:34",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "status": "final",
        "version": "2",
        "revision_history": [
            {
                "date": "2022-09-22T20:54:06",
                "number": "1",
                "summary": "Initial",
            },
            {
                "date": "2023-10-03T00:21:34",
                "number": "2",
                "summary": "Update",
            },
        ],
    }

    # deal with a missing revision history
    tracking = {
        "current_release_date": "2022-09-22T20:54:06.186927",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06.186927",
        "status": "final",
        "version": "",
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2022-09-22T20:54:06",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "revision_history": [
            {"date": "2022-09-22T20:54:06", "number": "1", "summary": "Initial"}
        ],
        "status": "final",
        "version": "1",
    }

    # deal with a NoneType revision history
    tracking = {
        "current_release_date": "2022-09-22T20:54:06.186927",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06.186927",
        "status": "final",
        "version": "",
        "revision_history": None,
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2022-09-22T20:54:06",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "revision_history": [
            {"date": "2022-09-22T20:54:06", "number": "1", "summary": "Initial"}
        ],
        "status": "final",
        "version": "1",
    }

    # update initial release date when adding initial release
    tracking = {
        "current_release_date": "2022-09-22T20:54:06.186927",
        "id": "ID",
        "initial_release_date": "2022-08-22T20:54:06.186927",
        "status": "final",
        "version": "1",
        "revision_history": [],
    }
    assert parse_revision_history(tracking) == {
        "current_release_date": "2022-09-22T20:54:06",
        "id": "ID",
        "initial_release_date": "2022-09-22T20:54:06",
        "revision_history": [
            {"date": "2022-09-22T20:54:06", "number": "1", "summary": "Initial"}
        ],
        "status": "final",
        "version": "1",
    }


def test_cleanup_list():
    assert cleanup_list([{}]) == []
    assert cleanup_list([{"a": "a", "b": "b", "c": ""}]) == [
        {"a": "a", "b": "b"}
    ]
    assert cleanup_list(["test", None]) == ["test"]


def test_cleanup_dict():
    assert cleanup_dict({"test": {"a": []}}) == {}
    assert cleanup_dict({"test": ""}) == {}
    assert cleanup_dict({"test": "", "test2": "test2"}) == {"test2": "test2"}
    assert cleanup_dict({"a": "a", "b": "b", "c": ""}) == {"a": "a", "b": "b"}
    assert cleanup_dict({"dict": ["test", None]}) == {"dict": ["test"]}


def test_get_ref_summary():
    url = "https://nvd.nist.gov/vuln/detail/cve-2021-1234"
    assert get_ref_summary(url) == "CVE Record"
    url = "https://github.com/user/repo/security/advisories/GHSA-1234-1234-1234"
    assert get_ref_summary(url) == "Advisory"
    url = "https://github.com/user/repo/pull/123"
    assert get_ref_summary(url) == "GitHub Pull Request"
    url = "https://github.com/user/repo/commit/123"
    assert get_ref_summary(url) == "GitHub Commit"
    url = ""
    assert get_ref_summary(url) == "Other"
    url = "https://example.com"
    assert get_ref_summary(url) == "Other"
    url = "https://github.com/user/repo/release"
    assert get_ref_summary(url) == "GitHub Repository Release"
    url = "https://github.com/user/repo"
    assert get_ref_summary(url) == "GitHub Repository"
    url = "https://access.redhat.com/security/cve/CVE-2023-26136"
    assert get_ref_summary(url) == "CVE Record"
    url = "https://access.redhat.com/errata/RHSA-2023:5484"
    assert get_ref_summary(url) == "Advisory"
    url = "https://bugzilla.redhat.com/show_bug.cgi?id=2224245"
    assert get_ref_summary(url) == "Bugzilla"


def test_format_references():
    advisories = [
        {
            "title": "testing",
            "url": "https://access.redhat.com/errata/RHSA-2023:5484",
        },
        {
            "title": "testing",
            "url": "https://bugzilla.redhat.com/show_bug.cgi?id=2224245",
        },
        {
            "title": "testing",
            "url": "https://nvd.nist.gov/vuln/detail/cve-2021-1234",
        },
        {
            "title": "testing",
            "url": "https://github.com/advisories/GHSA-1234-1234-1234",
        },
        {
            "title": "testing",
            "url": (
                "https://github.com/user/repo/security/advisories/GHSA"
                "-5432-5432-5432"
            ),
        },
        {"title": "testing", "url": "https://github.com/user/repo/pull/123"},
        {"title": "testing", "url": "https://github.com/user/repo/commit/123"},
        {"title": "testing", "url": "https://example.com"},
        {"title": "testing", "url": "https://github.com/user/repo/release"},
        {"title": "testing", "url": "https://github.com/user/repo"},
        {
            "title": "testing",
            "url": "https://bugzilla.redhat.com/show_bug.cgi?id=cve-2021-1234",
        },
        {
            "title": "testing",
            "url": "https://github.com/FasterXML/jackson-databind/issues/2816",
        },
        {
            "title": "testing",
            "url": (
                "https://sec.cloudapps.cisco.com/security/center/content"
                "/CiscoSecurityAdvisory/cisco-sa-apache-log4j-qRuKNEbd"
            ),
        },
        {
            "title": "testing",
            "url": "https://bitbucket.org/snakeyaml/snakeyaml/issues/525",
        },
        {
            "title": "testing",
            "url": (
                "https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=4707"
            ),
        },
    ]

    [ids, refs] = format_references(advisories)
    # For consistency in tests
    ids = sorted(ids, key=lambda x: x["text"])
    refs = sorted(refs, key=lambda x: x["url"])
    assert ids == [
        {"system_name": "Red Hat Bugzilla ID", "text": "2224245"},
        {
            "system_name": "GitHub Issue [FasterXML/jackson-databind]",
            "text": "2816",
        },
        {"system_name": "Chromium Issue [oss-fuzz]", "text": "4707"},
        {"system_name": "Bitbucket Issue [snakeyaml/snakeyaml]", "text": "525"},
        {"system_name": "GitHub Advisory", "text": "GHSA-1234-1234-1234"},
        {"system_name": "GitHub Advisory", "text": "GHSA-5432-5432-5432"},
        {"system_name": "Red Hat Advisory", "text": "RHSA-2023:5484"},
        {
            "system_name": "Cisco Advisory",
            "text": "cisco-sa-apache-log4j-qRuKNEbd",
        },
        {"system_name": "Red Hat Bugzilla ID", "text": "cve-2021-1234"},
    ]
    assert refs == [
        {
            "summary": "Red Hat Advisory",
            "url": "https://access.redhat.com/errata/RHSA-2023:5484",
        },
        {
            "summary": "Bitbucket Issue",
            "url": "https://bitbucket.org/snakeyaml/snakeyaml/issues/525",
        },
        {
            "summary": "Chromium Issue",
            "url": "https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=4707",
        },
        {
            "summary": "Red Hat Bugzilla",
            "url": "https://bugzilla.redhat.com/show_bug.cgi?id=2224245",
        },
        {
            "summary": "Red Hat Bugzilla",
            "url": "https://bugzilla.redhat.com/show_bug.cgi?id=cve-2021-1234",
        },
        {"summary": "Other", "url": "https://example.com"},
        {
            "summary": "GitHub Issue",
            "url": "https://github.com/FasterXML/jackson-databind/issues/2816",
        },
        {
            "summary": "GitHub Advisory",
            "url": "https://github.com/advisories/GHSA-1234-1234-1234",
        },
        {"summary": "GitHub Repository", "url": "https://github.com/user/repo"},
        {
            "summary": "GitHub Commit",
            "url": "https://github.com/user/repo/commit/123",
        },
        {
            "summary": "GitHub Pull Request",
            "url": "https://github.com/user/repo/pull/123",
        },
        {
            "summary": "GitHub Repository Release",
            "url": "https://github.com/user/repo/release",
        },
        {
            "summary": "GitHub Advisory",
            "url": (
                "https://github.com/user/repo/security/advisories/GHSA"
                "-5432-5432-5432"
            ),
        },
        {
            "summary": "CVE Record",
            "url": "https://nvd.nist.gov/vuln/detail/cve-2021-1234",
        },
        {
            "summary": "Cisco Advisory",
            "url": (
                "https://sec.cloudapps.cisco.com/security/center/content"
                "/CiscoSecurityAdvisory/cisco-sa-apache-log4j-qRuKNEbd"
            ),
        },
    ]


def test_parse_cwe():
    assert parse_cwe([20, 668]) == (
        {"id": "20", "name": "Improper Input Validation"},
        [
            {
                "title": "Additional CWE: 668",
                "audience": "developers",
                "category": "other",
                "text": "Exposure of Resource to Wrong Sphere",
            }
        ],
    )
    assert parse_cwe([1333]) == (
        {
            "id": "1333",
            "name": "Inefficient Regular Expression Complexity",
        },
        [],
    )
    assert parse_cwe([]) == (None, [])
    assert parse_cwe([1010101]) == (
        {"id": "1010101", "name": "UNABLE TO LOCATE CWE NAME"},
        [],
    )


def test_parse_toml():
    # If running tests using an IDE such as PyCharm, pytest may execute from
    # the test directory rather than the project root.
    if os.path.exists(os.path.join(os.getcwd(), "contrib/csaf.toml")):
        filepath = os.path.join(os.getcwd(), "contrib/csaf.toml")
    else:
        filepath = "../contrib/csaf.toml"
    metadata = import_csaf_toml(filepath)
    # We don't want a dynamically generated ID
    metadata["tracking"]["id"] = "1234"
    parsed_toml = parse_toml(metadata)
    assert parsed_toml["document"]["category"] == "csaf_vex"
    assert parsed_toml["document"]["notes"] == [
        {"audience": "", "category": "", "text": "", "title": ""}
    ]
    assert parsed_toml["document"]["publisher"] == {
        "category": "vendor",
        "contact_details": "vendor@mcvendorson.com",
        "name": "Vendor McVendorson",
        "namespace": "https://appthreat.com",
    }


def test_parse_cvss():
    assert parse_cvss([{
        "vector": "CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H"}]) == {
        'attackComplexity': 'LOW',
        'attackVector': 'NETWORK',
        'availabilityImpact': 'HIGH',
        'baseScore': 9.8,
        'baseSeverity': 'CRITICAL',
        'confidentialityImpact': 'HIGH',
        'environmentalScore': 9.8,
        'environmentalSeverity': 'CRITICAL',
        'integrityImpact': 'HIGH',
        'modifiedAttackComplexity': 'LOW',
        'modifiedAttackVector': 'NETWORK',
        'modifiedAvailabilityImpact': 'HIGH',
        'modifiedConfidentialityImpact': 'HIGH',
        'modifiedIntegrityImpact': 'HIGH',
        'modifiedPrivilegesRequired': 'NONE',
        'modifiedScope': 'UNCHANGED',
        'modifiedUserInteraction': 'NONE',
        'privilegesRequired': 'NONE',
        'scope': 'UNCHANGED',
        'temporalScore': 9.8,
        'temporalSeverity': 'CRITICAL',
        'userInteraction': 'NONE',
        'vectorString': 'CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H',
        'version': '3.0'
    }
    assert parse_cvss([{}]) == {}


def test_get_products():
    assert get_products([], []) == ([], {})
    affects = [
        {
            "ref": "pkg:maven/com.fasterxml.jackson.core/jackson-databind@2.9"
                   ".6?type=jar",
            "versions": [
                {"version": "2.9.6", "status": "affected"},
                {"version": "2.9.8", "status": "unaffected"},
            ]
        },
        {
            "ref": "pkg:maven/org.apache.httpcomponents/httpclient@4.5.5?type"
                   "=jar",
            "versions": [
                {"version": "4.5.5", "status": "affected"},
                {"version": "4.5.13", "status": "unaffected"},
            ]
        },
        {
            "ref": "pkg:pypi/setuptools@65.5.0",
            "versions": [
                {"version": "65.5.0", "status": "affected"},
            ]
        },
        {
            "ref": "pkg:golang/golang.org/x/net@v0.15.0",
            "versions": [{"version": "0.15.0", "status": "affected"}, ],
        },
            ]
    props = [
        {"name": "affectedVersionRange",
         "value": "com.fasterxml.jackson.core/jackson-databind@>=2.9.0-<2.9.8"},
        {"name": "affectedVersionRange",
         "value": "org.apache.httpcomponents/httpclient@>=4.5.0-<4.5.13"},
        {"name": "affectedVersionRange", "value": "setuptools@>=65.5.0"},
    ]
    [products, product_status] = get_products(affects, props)
    products.sort()
    assert products == [
        'com.fasterxml.jackson.core/jackson-databind@2.9.6',
        'com.fasterxml.jackson.core/jackson-databind@>=2.9.0-<2.9.8',
        'golang.org/x/net@v0.15.0',
        'org.apache.httpcomponents/httpclient@4.5.5',
        'org.apache.httpcomponents/httpclient@>=4.5.0-<4.5.13',
        'setuptools@65.5.0',
        'setuptools@>=65.5.0'
    ]
    assert product_status == {
        'fixed': ['com.fasterxml.jackson.core/jackson-databind@2.9.8',
           'org.apache.httpcomponents/httpclient@4.5.13'],
        'known_affected': [
            'com.fasterxml.jackson.core/jackson-databind@2.9.6',
            'org.apache.httpcomponents/httpclient@4.5.5',
            'setuptools@65.5.0',
            'golang.org/x/net@v0.15.0',
            'com.fasterxml.jackson.core/jackson-databind@>=2.9.0-<2.9.8',
            'org.apache.httpcomponents/httpclient@>=4.5.0-<4.5.13',
            'setuptools@>=65.5.0'
        ]}


def test_import_root_component():
    if os.path.exists("test/data/bom-root-comp.json"):
        [prod, ref] = import_root_component("test/data/bom-root-comp.json")
    else:
        [prod, ref] = import_root_component("data/bom-root-comp.json")

    assert prod == {
        "full_product_names": [
            {
                "name": "vuln-spring",
                "product_id": "vuln-spring:0.0.1-SNAPSHOT",
                "product_identification_helper": {
                    "purl": (
                        "pkg:maven/com.example/vuln-spring@0.0.1-SNAPSHOT"
                        "?type=jar"
                    )
                },
            }
        ]
    }

    assert ref == [
        {
            "summary": "website",
            "url": (
                "https://projects.spring.io/spring-boot/#/spring"
                "-boot-starter-parent/vuln-spring"
            ),
        },
        {
            "summary": "vcs",
            "url": (
                "https://github.com/spring-projects/spring-boot"
                "/spring-boot-starter-parent/vuln-spring"
            ),
        },
    ]


def test_verify_components_present():
    data = {
        "document": {
            "aggregate_severity": {"text": "Critical"},
            "category": "csaf_vex",
            "title": "Your Title",
            "csaf_version": "2.0",
            "lang": "en",
            "publisher": {
                "category": "vendor",
                "contact_details": "vendor@mcvendorson.com",
                "name": "Vendor McVendorson",
                "namespace": "https://appthreat.com",
            },
            "references": [
                {
                    "category": "self",
                    "summary": "dcksdnskljskl",
                    "url": "sdhasdjhslk.com",
                }
            ],
            "tracking": {
                "status": "draft",
                "initial_release_date": "2023-11-12T06:51:08",
                "current_release_date": "2023-11-12T06:51:08",
                "version": "1",
                "id": "2023-11-22T10:35:03_v1",
            },
        },
        "vulnerabilities": [],
    }
    metadata = {
        "depscan_version": "5.0.2",
        "note": [
            {"audience": "", "category": "", "text": "", "title": ""},
            {"audience": "", "category": "", "text": "", "title": ""},
        ],
        "reference": [
            {
                "category": "self",
                "summary": "dcksdnskljskl",
                "url": "sdhasdjhslk.com",
            },
            {"category": "", "summary": "", "url": ""},
        ],
        "distribution": {"label": "", "text": "", "url": ""},
        "document": {"category": "csaf_vex", "title": "Your Title"},
        "product_tree": {"easy_import": ""},
        "publisher": {
            "category": "vendor",
            "contact_details": "vendor@mcvendorson.com",
            "name": "Vendor McVendorson",
            "namespace": "https://appthreat.com",
        },
        "tracking": {
            "status": "draft",
            "initial_release_date": "2023-11-12T06:51:08",
            "current_release_date": "2023-11-12T06:51:08",
            "version": "1",
            "id": "",
        },
    }
    if os.path.exists("test/data/bom-root-comp.json"):
        vdr_file = "test/data/bom-root-comp.json"
    else:
        vdr_file = "data/bom-root-comp.json"

    [template, new_metadata] = verify_components_present(
        data, metadata, vdr_file
    )
    assert template["document"]["notes"] == [
        {
            "category": "legal_disclaimer",
            "text": (
                "Depscan reachable code only covers the "
                "project source code, not the code of "
                "dependencies. A dependency may execute "
                "vulnerable code when called even if it is "
                "not in the project's source code. Regard the "
                "Depscan-set flag of "
                "'code_not_in_execute_path' with this in "
                "mind."
            ),
        }
    ]
    assert template["product_tree"] == {
        "full_product_names": [
            {
                "name": "vuln-spring",
                "product_id": "vuln-spring:0.0.1-SNAPSHOT",
                "product_identification_helper": {
                    "purl": "pkg:maven/com.example/vuln-spring@0.0.1-SNAPSHOT"
                            "?type=jar"
                },
            }
        ]
    }
    assert new_metadata["tracking"] == {
        "current_release_date": "2023-11-12T06:51:08",
        "id": "",
        "initial_release_date": "2023-11-12T06:51:08",
        "status": "draft",
        "version": "1",
    }


def test_add_vulnerabilities():
    template = {
        "document": {
            "aggregate_severity": {},
            "category": "csaf_vex",
            "title": "Your Title",
            "csaf_version": "2.0",
            "distribution": {"label": "", "text": "", "url": ""},
            "lang": "en",
            "notes": [
                {"audience": "", "category": "", "text": "", "title": ""},
                {"audience": "", "category": "", "text": "", "title": ""},
            ],
            "publisher": {
                "category": "vendor",
                "contact_details": "vendor@mcvendorson.com",
                "name": "Vendor McVendorson",
                "namespace": "https://appthreat.com",
            },
            "references": [
                {
                    "category": "self",
                    "summary": "dcksdnskljskl",
                    "url": "sdhasdjhslk.com",
                },
                {"category": "", "summary": "", "url": ""},
            ],
            "tracking": {
                "status": "draft",
                "initial_release_date": "2023-11-12T06:51:08",
                "current_release_date": "2023-11-12T06:51:08",
                "version": "1",
                "id": "2023-11-21T21:39:14_v1",
                "revision_history": [],
            },
        },
        "product_tree": None,
        "vulnerabilities": [],
    }
    pkg_vulnerabilities = [
        {
            "bom-ref": "CVE-2020-10673/pkg:maven/com.fasterxml.jackson.core"
                       "/jackson-databind@2.9.6?type=jar",
            "id": "CVE-2020-10673",
            "published": "2020-03-18T22:15:00",
            "updated": "2023-11-07T03:14:00",
            "source": {
                "name": "NVD",
                "url": "https://nvd.nist.gov/vuln/detail/CVE-2020-10673",
            },
            "ratings": [
                {
                    "score": 8.8,
                    "severity": "high",
                    "method": "CVSSv31",
                    "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H"
                }
            ],
            "cwes": [668, 520, 521],
            "description": (
                "FasterXML jackson-databind 2.x before 2.9.10.4 mishandles the"
                " interaction between serialization gadgets and typing, related"
                " to com.caucho.config.types.ResourceRef (aka caucho-quercus)."
            ),
            "recommendation": "Update to 2.12.7.1 or later",
            "advisories": [
                {
                    "title": "GitHub Issue",
                    "url": "https://github.com/FasterXML/jackson-databind"
                           "/issues/2660",
                },
                {
                    "title": "Mailing List",
                    "url": "https://lists.debian.org/debian-lts-announce/2020"
                           "/03/msg00027.html",
                },
                {
                    "title": "vendor",
                    "url": (
                        "https://www.oracle.com/security-alerts/cpuoct2021.html"
                    ),
                },
            ],
            "affects": [
                {
                    "ref": "pkg:maven/com.fasterxml.jackson.core/jackson"
                           "-databind@2.9.6?type=jar",
                    "versions": [
                        {"version": "2.9.6", "status": "affected"},
                        {"version": "2.12.7.1", "status": "unaffected"},
                    ],
                }
            ],
            "properties": [
                {
                    "name": "depscan:insights",
                    "value": "Direct dependency\\nVendor Confirmed",
                },
                {"name": "depscan:prioritized", "value": "true"},
                {
                    "name": "affected_version_range",
                    "value": ">=2.9.0-<2.9.10.4",
                },
            ],
        },
        {
            "bom-ref": "CVE-2021-20190/pkg:maven/com.fasterxml.jackson.core"
                       "/jackson-databind@2.9.6?type=jar",
            "id": "CVE-2021-20190",
            "published": "2021-01-19T17:15:00",
            "updated": "2023-11-07T03:28:00",
            "source": {
                "name": "NVD",
                "url": "https://nvd.nist.gov/vuln/detail/CVE-2021-20190",
            },
            "ratings": [
                {
                    "score": 8.1,
                    "severity": "high",
                    "method": "CVSSv31",
                    "vector": "CVSS:3.1/AV:N/AC:H/PR:N/UI:N/S:U/C:H/I:H/A:H"
                }
            ],
            "cwes": [502],
            "description": (
                "A flaw was found in jackson-databind before 2.9.10.7."
                " FasterXML mishandles the interaction between serialization"
                " gadgets and typing. The highest threat from this"
                " vulnerability is to data confidentiality and integrity as"
                " well as system availability."
            ),
            "recommendation": "Update to 2.12.7.1 or later",
            "advisories": [
                {
                    "title": "GitHub Issue",
                    "url": "https://github.com/FasterXML/jackson-databind"
                           "/issues/2854",
                },
                {
                    "title": "vendor",
                    "url": "https://www.oracle.com//security-alerts"
                           "/cpujul2021.html",
                },
                {
                    "title": "Mailing List",
                    "url": "https://lists.debian.org/debian-lts-announce/2021"
                           "/04/msg00025.html",
                },
            ],
            "affects": [
                {
                    "ref": "pkg:maven/com.fasterxml.jackson.core/jackson"
                           "-databind@2.9.6?type=jar",
                    "versions": [
                        {"version": "2.9.6", "status": "affected"},
                        {"version": "2.12.7.1", "status": "unaffected"},
                    ],
                }
            ],
            "properties": [
                {
                    "name": "depscan:insights",
                    "value": "Direct dependency\\nVendor Confirmed",
                },
                {"name": "depscan:prioritized", "value": "true"},
                {
                    "name": "affected_version_range",
                    "value": ">=2.7.0-<2.9.10.7",
                },
            ],
        },
    ]
    new_results = add_vulnerabilities(template, pkg_vulnerabilities)
    assert new_results.get("vulnerabilities") == [
        {
            "acknowledgements": {
                "organization": "NVD",
                "urls": ["https://nvd.nist.gov/vuln/detail/CVE-2020-10673"],
            },
            "cve": "CVE-2020-10673",
            "cwe": {
                "id": "668",
                "name": "Exposure of Resource to Wrong Sphere",
            },
            "discovery_date": "2020-03-18T22:15:00",
            "ids": [
                {
                    "system_name": "GitHub Issue [FasterXML/jackson-databind]",
                    "text": "2660",
                }
            ],
            "notes": [
                {
                    "audience": "developers",
                    "category": "other",
                    "text": ".NET Misconfiguration: Use of Impersonation",
                    "title": "Additional CWE: 520",
                },
                {
                    "audience": "developers",
                    "category": "other",
                    "text": "Weak Password Requirements",
                    "title": "Additional CWE: 521",
                },
                {
                    "category": "general",
                    "details": "Vulnerability Description",
                    "text": (
                        "FasterXML jackson-databind 2.x before 2.9.10.4"
                        " mishandles the interaction between serialization"
                        " gadgets and typing, related to"
                        " com.caucho.config.types.ResourceRef (aka"
                        " caucho-quercus)."
                    ),
                },
            ],
            "product_status": {
                "fixed": [
                    "com.fasterxml.jackson.core/jackson-databind@2.12.7.1"
                ],
                "known_affected": [
                    "com.fasterxml.jackson.core/jackson-databind@2.9.6"
                ],
            },
            "references": [
                {
                    "summary": "Mailing List Announcement",
                    "url": "https://lists.debian.org/debian-lts-announce/2020"
                           "/03/msg00027.html",
                },
                {
                    "summary": "Oracle Security Alert",
                    "url": (
                        "https://www.oracle.com/security-alerts/cpuoct2021.html"
                    ),
                },
                {
                    "summary": "GitHub Issue",
                    "url": "https://github.com/FasterXML/jackson-databind"
                           "/issues/2660",
                },
            ],
            "scores": [
                {'cvss_v3': {
                    'attackComplexity': 'LOW',
                    'attackVector': 'NETWORK',
                    'availabilityImpact': 'HIGH',
                    'baseScore': 8.8,
                    'baseSeverity': 'HIGH',
                    'confidentialityImpact': 'HIGH',
                    'environmentalScore': 8.8,
                    'environmentalSeverity': 'HIGH',
                    'integrityImpact': 'HIGH',
                    'modifiedAttackComplexity': 'LOW',
                    'modifiedAttackVector': 'NETWORK',
                    'modifiedAvailabilityImpact': 'HIGH',
                    'modifiedConfidentialityImpact': 'HIGH',
                    'modifiedIntegrityImpact': 'HIGH',
                    'modifiedPrivilegesRequired': 'NONE',
                    'modifiedScope': 'UNCHANGED',
                    'modifiedUserInteraction': 'REQUIRED',
                    'privilegesRequired': 'NONE',
                    'scope': 'UNCHANGED',
                    'temporalScore': 8.8,
                    'temporalSeverity': 'HIGH',
                    'userInteraction': 'REQUIRED',
                    'vectorString': 'CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:H/I:H'
                                    '/A:H',
                    'version': '3.1'
                },
                    "products": [
                        "com.fasterxml.jackson.core/jackson-databind@2.9.6"
                    ],
                }
            ],
        },
        {
            "acknowledgements": {
                "organization": "NVD",
                "urls": ["https://nvd.nist.gov/vuln/detail/CVE-2021-20190"],
            },
            "cve": "CVE-2021-20190",
            "cwe": {"id": "502", "name": "Deserialization of Untrusted Data"},
            "discovery_date": "2021-01-19T17:15:00",
            "ids": [
                {
                    "system_name": "GitHub Issue [FasterXML/jackson-databind]",
                    "text": "2854",
                }
            ],
            "notes": [
                {
                    "category": "general",
                    "details": "Vulnerability Description",
                    "text": (
                        "A flaw was found in jackson-databind before "
                        "2.9.10.7. FasterXML mishandles the interaction "
                        "between serialization gadgets and typing. The "
                        "highest threat from this vulnerability is to "
                        "data confidentiality and integrity as well as "
                        "system availability."
                    ),
                }
            ],
            "product_status": {
                "fixed": [
                    "com.fasterxml.jackson.core/jackson-databind@2.12.7.1"
                ],
                "known_affected": [
                    "com.fasterxml.jackson.core/jackson-databind@2.9.6"
                ],
            },
            "references": [
                {
                    "summary": "Other",
                    "url": (
                        "https://www.oracle.com//security-alerts"
                        "/cpujul2021.html"
                    ),
                },
                {
                    "summary": "Mailing List Announcement",
                    "url": (
                        "https://lists.debian.org/debian-lts-announce/2021"
                        "/04/msg00025.html"
                    ),
                },
                {
                    "summary": "GitHub Issue",
                    "url": (
                        "https://github.com/FasterXML/jackson-databind"
                        "/issues/2854"
                    ),
                },
            ],
            "scores": [
                {
                    'cvss_v3': {
                        'attackComplexity': 'HIGH',
                        'attackVector': 'NETWORK',
                        'availabilityImpact': 'HIGH',
                        'baseScore': 8.1,
                        'baseSeverity': 'HIGH',
                        'confidentialityImpact': 'HIGH',
                        'environmentalScore': 8.1,
                        'environmentalSeverity': 'HIGH',
                        'integrityImpact': 'HIGH',
                        'modifiedAttackComplexity': 'HIGH',
                        'modifiedAttackVector': 'NETWORK',
                        'modifiedAvailabilityImpact': 'HIGH',
                        'modifiedConfidentialityImpact': 'HIGH',
                        'modifiedIntegrityImpact': 'HIGH',
                        'modifiedPrivilegesRequired': 'NONE',
                        'modifiedScope': 'UNCHANGED',
                        'modifiedUserInteraction': 'NONE',
                        'privilegesRequired': 'NONE',
                        'scope': 'UNCHANGED',
                        'temporalScore': 8.1,
                        'temporalSeverity': 'HIGH',
                        'userInteraction': 'NONE',
                        'vectorString': 'CVSS:3.1/AV:N/AC:H/PR:N/UI:N/S:U/C:H'
                                        '/I:H/A:H',
                        'version': '3.1'
                    },
                    "products": [
                        "com.fasterxml.jackson.core/jackson-databind@2.9.6"
                    ],
                }
            ],
        },
    ]
    new_results = add_vulnerabilities(template, [])
    assert new_results.get("vulnerabilities") == []


def test_get_acknowledgements():
    source = {
        "name": "NVD",
        "url": "https://nvd.nist.gov/vuln/detail/CVE-2021-20190",
    }
    assert get_acknowledgements(source) == {
        "organization": "NVD",
        "urls": ["https://nvd.nist.gov/vuln/detail/CVE-2021-20190"],
    }
    source = {"url": "https://nvd.nist.gov/vuln/detail/CVE-2021-20190"}
    assert get_acknowledgements(source) == {}




import json
import os
import shutil
import subprocess
import sys
from urllib.parse import unquote_plus

import httpx
from defusedxml.ElementTree import parse

from depscan.lib.logger import LOG
from depscan.lib.utils import cleanup_license_string, find_files

headers = {
    "Content-Type": "application/json",
    "Accept-Encoding": "gzip",
}


def exec_tool(args, cwd=None, stdout=subprocess.PIPE):
    """
    Convenience method to invoke cli tools

    :param args: Command line arguments
    :param cwd: Working directory
    :param stdout: Specifies stdout of command
    """
    try:
        LOG.debug('⚡︎ Executing "%s"', " ".join(args))
        cp = subprocess.run(
            args,
            stdout=stdout,
            stderr=subprocess.STDOUT,
            cwd=cwd,
            env=os.environ.copy(),
            shell=sys.platform == "win32",
            encoding="utf-8",
            check=False,
        )
        LOG.debug(cp.stdout)
    except Exception as e:
        LOG.exception(e)


def parse_bom_ref(bomstr, licenses=None):
    """
    Method to parse bom ref string into individual constituents

    :param bomstr: Bom ref string
    :param licenses: Licenses
    :return Dict containing group, name, and version for the package
    """
    if bomstr:
        bomstr = unquote_plus(bomstr)
    tmpl = bomstr.split("/")
    vendor = ""
    name_ver = []
    if len(tmpl) == 2:
        # Just name and version
        vendor = tmpl[0]
        name_ver = tmpl[1].split("@")
    elif len(tmpl) == 3:
        vendor = tmpl[1]
        name_ver = tmpl[-1].split("@")
    elif len(tmpl) > 3:
        vendor = tmpl[-2]
        name_ver = tmpl[-1].split("@")
    vendor = vendor.replace("pkg:", "")
    # If name starts with @ this will make sure the name still gets captured
    if len(name_ver) >= 2:
        name = name_ver[-2]
        version = name_ver[-1]
    else:
        name = name_ver[0]
        version = "*"
    if "?" in version:
        version = version.split("?")[0]
    if version.startswith("v"):
        version = version[1:]
    return {
        "vendor": vendor,
        "name": name,
        "version": version,
        "licenses": licenses,
    }


def get_licenses(ele):
    """
    Retrieve licenses from xml

    :param ele: An XML element
    :return A list of extracted licenses
    """
    license_list = []
    namespace = "{http://cyclonedx.org/schema/bom/1.5}"
    for data in ele.findall(
        f"{namespace}licenses/{namespace}license/{namespace}id"
    ):
        license_list.append(data.text)
    if not license_list:
        for data in ele.findall(
            f"{namespace}licenses/{namespace}license/{namespace}name"
        ):
            if data and data.text:
                ld_list = [data.text]
                if "http" in data.text:
                    ld_list = [
                        os.path.basename(data.text)
                        .replace(".txt", "")
                        .replace(".html", "")
                    ]
                elif "/" in data.text:
                    ld_list = [cleanup_license_string(data.text)]
                for ld in ld_list:
                    license_list.append(ld.strip().upper())
    return license_list


def get_package(component_ele, licenses):
    """
    Retrieve package from xml

    :param component_ele: The XML element representing a component.
    :param licenses: A list of licenses associated with the component.
    :return: A dictionary containing the package information
    """
    bom_ref = component_ele.attrib.get("bom-ref")
    pkg = {
        "licenses": licenses,
        "vendor": "",
        "name": "",
        "version": "",
        "scope": "",
    }
    if bom_ref and "/" in bom_ref:
        pkg = parse_bom_ref(bom_ref, licenses)
    for ele in component_ele.iter():
        if ele.tag.endswith("group") and ele.text:
            pkg["vendor"] = ele.text
        if ele.tag.endswith("name") and ele.text and not pkg["name"]:
            pkg["name"] = ele.text
        if ele.tag.endswith("version") and ele.text:
            version = ele.text
            if version.startswith("v"):
                version = version[1:]
            pkg["version"] = version
        if ele.tag.endswith("purl") and ele.text and not pkg.get("vendor"):
            purl = ele.text
            namespace = purl.split("/")[0].replace("pkg:", "")
            pkg["vendor"] = namespace
    return pkg


def get_pkg_list_json(jsonfile):
    """
    Method to extract packages from a bom json file

    :param jsonfile: Path to a bom json file.
    return List of dicts representing extracted packages
    """
    pkgs = []
    with open(jsonfile, encoding="utf-8") as fp:
        try:
            bom_data = json.load(fp)
            if bom_data and bom_data.get("components"):
                for comp in bom_data.get("components"):
                    licenses = []
                    vendor = comp.get("group")
                    if not vendor:
                        vendor = ""
                    if comp.get("licenses"):
                        for lic in comp.get("licenses"):
                            license_obj = lic
                            # licenses has list of dict with either license
                            # or expression as key Only license is supported
                            # for now
                            if lic.get("license"):
                                license_obj = lic.get("license")
                            if license_obj.get("id"):
                                licenses.append(license_obj.get("id"))
                            elif license_obj.get("name"):
                                licenses.append(
                                    cleanup_license_string(
                                        license_obj.get("name")
                                    )
                                )
                    pkgs.append(
                        {**comp, "vendor": vendor, "licenses": licenses}
                    )
        except Exception:
            # Ignore json errors
            pass
        return pkgs


def get_pkg_list(xmlfile):
    """Method to parse the bom xml file and convert into packages list

    :param xmlfile: BOM xml file to parse
    :return list of package dict
    """
    if xmlfile.endswith(".json"):
        return get_pkg_list_json(xmlfile)
    pkgs = []
    try:
        et = parse(xmlfile)
        root = et.getroot()
        for child in root:
            if child.tag.endswith("components"):
                for ele in child.iter():
                    if ele.tag.endswith("component"):
                        licenses = get_licenses(ele)
                        pkgs.append(get_package(ele, licenses))
    except Exception as pe:
        LOG.debug("Unable to parse %s %s", xmlfile, pe)
        LOG.warning(
            "Unable to produce Software Bill-of-Materials for this project. "
            "Execute the scan after installing the dependencies!"
        )
    return pkgs


def get_pkg_by_type(pkg_list, pkg_type):
    """Method to filter packages based on package type

    :param pkg_list: List of packages
    :param pkg_type: Package type to filter
    :return List of packages matching pkg_type
    """
    if not pkg_list:
        return []
    return [
        pkg
        for pkg in pkg_list
        if pkg.get("purl", "").startswith("pkg:" + pkg_type)
    ]


def resource_path(relative_path):
    """
    Determine the absolute path of a resource file based on its relative path.

    :param relative_path: Relative path of the resource file.
    :return: Absolute path of the resource file
    """
    try:
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.dirname(__file__)
    return os.path.join(base_path, relative_path)


def exec_cdxgen(use_bin=True):
    if use_bin:
        cdxgen_cmd = os.environ.get("CDXGEN_CMD", "cdxgen")
        if not shutil.which(cdxgen_cmd):
            local_bin = resource_path(
                os.path.join(
                    "local_bin",
                    "cdxgen.exe" if sys.platform == "win32" else "cdxgen",
                )
            )
            if not os.path.exists(local_bin):
                LOG.warning(
                    "%s command not found. Please install using npm install "
                    "@cyclonedx/cdxgen or set PATH variable",
                    cdxgen_cmd,
                )
                return False
            try:
                cdxgen_cmd = local_bin
                # Set the plugins directory as an environment variable
                os.environ["CDXGEN_PLUGINS_DIR"] = resource_path("local_bin")
                return cdxgen_cmd
            except Exception:
                return None
        else:
            return cdxgen_cmd
    else:
        lbin = os.getenv("APPDATA") if sys.platform == "win32" else "local_bin"
        local_bin = resource_path(
            os.path.join(
                f"{lbin}\\npm\\" if sys.platform == "win32" else "local_bin",
                "cdxgen" if sys.platform != "win32" else "cdxgen.cmd",
            )
        )
        if not os.path.exists(local_bin):
            LOG.warning(
                "%s command not found. Please install using npm install "
                "@cyclonedx/cdxgen or set PATH variable",
                local_bin,
            )
            return None
        try:
            cdxgen_cmd = local_bin
            # Set the plugins directory as an environment variable
            os.environ["CDXGEN_PLUGINS_DIR"] = (
                resource_path("local_bin")
                if sys.platform != "win32"
                else resource_path(
                    os.path.join(
                        lbin,
                        "\\npm\\node_modules\\@cyclonedx\\cdxgen\\node_modules\\@cyclonedx\\cdxgen-plugins-bin\\plugins",
                    )
                )
            )
            return cdxgen_cmd
        except Exception:
            return None


def create_bom(project_type, bom_file, src_dir=".", deep=False, options={}):
    """
    Method to create BOM file by executing cdxgen command

    :param project_type: Project type
    :param bom_file: BOM file
    :param src_dir: Source directory
    :param deep: A boolean flag indicating whether to perform a deep scan.
    :param options: Additional options for generating the BOM file.
    :returns: True if the command was executed. False if the executable was
    not found.
    """
    cdxgen_server = options.get("cdxgen_server")
    # Generate SBOM by calling cdxgen server
    if cdxgen_server:
        # Fallback to universal if no project type was provided
        if not project_type:
            project_type = "universal"
        if not src_dir and options.get("path"):
            src_dir = options.get("path")
        with httpx.Client(
            http2=True, base_url=cdxgen_server, timeout=180
        ) as client:
            sbom_url = f"{cdxgen_server}/sbom"
            LOG.debug("Invoking cdxgen server at %s", sbom_url)
            try:
                r = client.post(
                    sbom_url,
                    json={
                        "url": options.get("url", ""),
                        "path": options.get("path", src_dir),
                        "type": options.get("type", project_type),
                        "multiProject": options.get("multiProject", ""),
                    },
                    headers=headers,
                )
                if r.status_code == httpx.codes.OK:
                    try:
                        json_response = r.json()
                        if json_response:
                            with open(
                                bom_file, mode="w", encoding="utf-8"
                            ) as fp:
                                json.dump(json_response, fp)
                            return os.path.exists(bom_file)
                    except Exception as je:
                        LOG.error(je)
                        LOG.info(
                            "Unable to generate SBOM with cdxgen server. "
                            "Trying to generate one locally."
                        )
                else:
                    LOG.warning(
                        "Unable to generate SBOM via cdxgen server due to %s",
                        r.status_code,
                    )
            except Exception as e:
                LOG.error(e)
                LOG.info(
                    "Unable to generate SBOM with cdxgen server. Trying to "
                    "generate one locally."
                )
    cdxgen_cmd = exec_cdxgen()
    if not cdxgen_cmd:
        cdxgen_cmd = exec_cdxgen(False)
    if project_type in ("docker",):
        LOG.info(
            "Generating Software Bill-of-Materials for container image %s. "
            "This might take a few mins ...",
            src_dir,
        )
    args = [cdxgen_cmd, "-r", "-t", project_type, "-o", bom_file]
    if deep or project_type in ("jar", "jenkins") or options.get("reachables"):
        args.append("--deep")
        LOG.info("About to perform deep scan. This could take a while ...")
    if options.get("profile"):
        args.append("--profile")
        args.append(options.get("profile"))
        if options.get("profile") != "generic":
            LOG.debug("BOM Profile: %s", options.get("profile"))
    # Bug #233 - Source directory could be None when working with url
    if src_dir:
        args.append(src_dir)
    if cdxgen_cmd:
        exec_tool(
            args,
            src_dir
            if project_type not in ("docker", "oci", "container")
            and src_dir
            and os.path.isdir(src_dir)
            else None,
        )
    else:
        LOG.warning("Unable to locate cdxgen command. ")
    return os.path.exists(bom_file)


def submit_bom(reports_dir, threatdb_params):
    """
    Method to submit the SBOM to threatdb for analysis

    :param reports_dir: The directory where the SBOM reports are located.
    :param threatdb_params: A dict of threatdb parameters
    """
    vdr_files = find_files(reports_dir, ".vdr.json")
    if vdr_files:
        threatdb_server = threatdb_params["threatdb_server"]
        if not threatdb_server.endswith("/import"):
            threatdb_server = f"{threatdb_server}/import"
        login_url = threatdb_server.replace("/import", "/login")
        with httpx.Client(
            http2=True, base_url=threatdb_server, timeout=180
        ) as client:
            token = threatdb_params.get("threatdb_token")
            if not token:
                LOG.debug(
                    "Attempting to retrieve access token from %s", login_url
                )
                r = client.post(
                    login_url,
                    json={
                        "username": threatdb_params["threatdb_username"],
                        "password": threatdb_params["threatdb_password"],
                    },
                    headers=headers,
                )
                if r.status_code == httpx.codes.OK:
                    json_response = r.json()
                    if json_response and json_response.get("access_token"):
                        token = json_response.get("access_token")
                else:
                    LOG.warning(
                        "Unable to retrieve access token from %s due to %s",
                        login_url,
                        r.status_code,
                    )
            if token:
                for vf in vdr_files:
                    files = {"file": open(vf, "rb")}
                    r = client.post(
                        threatdb_server,
                        files=files,
                        headers={"Authorization": f"Bearer {token}"},
                    )
                    if r.status_code == httpx.codes.OK:
                        json_response = r.json()
                        if not json_response.get("success"):
                            LOG.debug(
                                "Uploaded file %s was not processed successfully",
                                vf,
                            )
                        else:
                            LOG.debug(
                                "VDR file %s was submitted successfully to "
                                "the threatdb server",
                                vf,
                            )
                    else:
                        LOG.warning(
                            "Unable to submit VDR file to %s due to %s",
                            threatdb_server,
                            r.status_code,
                        )



import json
import os

import pytest

from depscan.lib import config as config
from depscan.lib.bom import get_pkg_list
from depscan.lib.pkg_query import (
    calculate_risk_score,
    get_category_score,
    npm_metadata,
    npm_pkg_risk,
    pypi_metadata,
    pypi_pkg_risk,
)


def test_risk_scores():
    cat_score = get_category_score(2, 100, 2)
    assert cat_score > 0.4
    cat_score = get_category_score(1000, config.mod_create_min_seconds_max, 1)
    assert cat_score > 0.3
    cat_score = get_category_score(120, config.latest_now_min_seconds_max, 0.5)
    assert cat_score > 0.1


def test_calculate_risk_score():
    # Test for min versions
    one_score = calculate_risk_score(
        {"pkg_min_versions_risk": True, "pkg_min_versions_value": 1}
    )
    assert one_score > 0
    two_score = calculate_risk_score(
        {"pkg_min_versions_risk": True, "pkg_min_versions_value": 2}
    )
    assert two_score < one_score
    # Deprecated package risk
    dep_score = calculate_risk_score({"pkg_deprecated_risk": True})
    assert dep_score > one_score
    dep_score = calculate_risk_score(
        {
            "pkg_deprecated_risk": True,
            "pkg_min_versions_risk": True,
            "pkg_min_versions_value": 1,
        }
    )
    assert dep_score > one_score
    # Min maintainers risk
    m_score = calculate_risk_score(
        {
            "pkg_min_maintainers_risk": True,
            "pkg_min_maintainers_value": 1,
            "pkg_min_versions_risk": True,
            "pkg_min_versions_value": 1,
        }
    )
    assert m_score > two_score
    # Recent package
    l1_score = calculate_risk_score(
        {
            "pkg_min_versions_risk": True,
            "pkg_min_versions_value": 1,
            "created_now_quarantine_seconds_risk": True,
            "latest_now_max_seconds_risk": True,
            "latest_now_max_seconds_value": 1000,
        }
    )
    # Recent package with less maintainers (risky)
    l2_score = calculate_risk_score(
        {
            "pkg_min_maintainers_risk": True,
            "pkg_min_maintainers_value": 1,
            "pkg_min_versions_risk": True,
            "pkg_min_versions_value": 1,
            "created_now_quarantine_seconds_risk": True,
            "latest_now_max_seconds_risk": True,
            "latest_now_max_seconds_value": 1000,
        }
    )
    assert l2_score > l1_score
    assert l2_score > 0.5
    # Also has script section
    l3_score = calculate_risk_score(
        {
            "pkg_min_maintainers_risk": True,
            "pkg_min_maintainers_value": 1,
            "pkg_min_versions_risk": True,
            "pkg_min_versions_value": 1,
            "created_now_quarantine_seconds_risk": True,
            "latest_now_max_seconds_risk": True,
            "latest_now_max_seconds_value": 1000,
            "pkg_install_scripts_risk": True,
            "pkg_install_scripts_value": 1,
        }
    )
    assert l3_score > l2_score

    # Old package with scripts but past quarantine
    l4_score = calculate_risk_score(
        {
            "pkg_min_maintainers_risk": True,
            "pkg_min_maintainers_value": 1,
            "pkg_min_versions_risk": True,
            "created_now_quarantine_seconds_risk": False,
            "created_now_quarantine_seconds_value": 2 * 365 * 60 * 60,
            "pkg_min_versions_value": 1,
            "pkg_install_scripts_risk": True,
            "pkg_install_scripts_value": 1,
        }
    )
    assert l4_score < l3_score

    # Quite old package
    o_score = calculate_risk_score(
        {
            "latest_now_max_seconds_risk": True,
            "latest_now_max_seconds_value": 10 * 365 * 60 * 60,
            "pkg_node_version_risk": True,
            "pkg_node_version_value": 1,
        }
    )
    assert o_score < 0.2
    od_score = calculate_risk_score(
        {
            "latest_now_max_seconds_risk": True,
            "latest_now_max_seconds_value": 10 * 365 * 60 * 60,
            "pkg_node_version_risk": True,
            "pkg_node_version_value": 1,
            "pkg_deprecated_risk": True,
        }
    )
    assert od_score > o_score


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_query_metadata():
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-node.xml"
    )
    pkg_list = get_pkg_list(test_bom)
    metadata_dict = npm_metadata({}, pkg_list, None)
    assert metadata_dict


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_query_metadata1():
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-goof.json"
    )
    pkg_list = get_pkg_list(test_bom)
    metadata_dict = npm_metadata({}, pkg_list, "snyk")
    assert metadata_dict


def test_npm_confusion_risks():
    test_deprecated_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "cdxgen-metadata.json"
    )
    with open(test_deprecated_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = npm_pkg_risk(pkg_metadata, True, None)
        assert risk_metrics["pkg_private_on_public_registry_risk"]
        assert not risk_metrics["pkg_min_versions_risk"]


def test_npm_risks():
    test_deprecated_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bcrypt-metadata.json"
    )
    with open(test_deprecated_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = npm_pkg_risk(pkg_metadata, False, None)
        assert risk_metrics["pkg_deprecated_risk"]
        assert not risk_metrics["pkg_min_versions_risk"]
        assert risk_metrics["latest_now_max_seconds_risk"]

    ebp_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "ebparser-metadata.json"
    )
    with open(ebp_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = npm_pkg_risk(pkg_metadata, False, None)
        assert risk_metrics["pkg_node_version_risk"]
        assert not risk_metrics["pkg_deprecated_risk"]
        assert not risk_metrics["pkg_min_versions_risk"]


def test_pypi_confusion_risks():
    test_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "django-metadata.json"
    )
    with open(test_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = pypi_pkg_risk(pkg_metadata, False, None)
        assert risk_metrics == {
            "pkg_deprecated_risk": False,
            "pkg_min_versions_risk": False,
            "created_now_quarantine_seconds_risk": False,
            "latest_now_max_seconds_risk": False,
            "mod_create_min_seconds_risk": False,
            "pkg_min_maintainers_risk": False,
            "pkg_private_on_public_registry_risk": False,
            "risk_score": 0.0,
        }
    test_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "astroid-metadata.json"
    )
    with open(test_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = pypi_pkg_risk(pkg_metadata, False, None)
        assert risk_metrics == {
            "pkg_deprecated_risk": False,
            "pkg_min_versions_risk": False,
            "created_now_quarantine_seconds_risk": False,
            "latest_now_max_seconds_risk": False,
            "mod_create_min_seconds_risk": False,
            "pkg_min_maintainers_risk": False,
            "pkg_private_on_public_registry_risk": False,
            "risk_score": 0.0,
        }
    test_pkg = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "mongo-dash-metadata.json"
    )
    with open(test_pkg) as fp:
        pkg_metadata = json.load(fp)
        risk_metrics = pypi_pkg_risk(pkg_metadata, False, None)
        assert risk_metrics
        assert risk_metrics["pkg_min_versions_risk"]


def test_query_metadata2():
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-py.xml"
    )
    pkg_list = get_pkg_list(test_bom)
    metadata_dict = pypi_metadata({}, pkg_list, None)
    assert metadata_dict



import os
import shutil
import subprocess
import tarfile
import tempfile

import oras.client
import oras.provider
from oras.logger import setup_logger
from vdb.lib.config import data_dir

from depscan.lib.config import vdb_database_url, vdb_rafs_database_url
from depscan.lib.logger import LOG

setup_logger(quiet=True, debug=False)


class VdbDistributionRegistry(oras.provider.Registry):
    """
    We override the default registry to make things compatible with ghcr. Without this, the below error is thrown.

    jsonschema.exceptions.ValidationError: Additional properties are not allowed ('artifactType' was unexpected)
    """

    def get_manifest(self, container, allowed_media_type=None):
        """
        Retrieve a manifest for a package.

        :param container:  parsed container URI
        :type container: oras.container.Container or str
        :param allowed_media_type: one or more allowed media types
        :type allowed_media_type: str
        """
        if not allowed_media_type:
            allowed_media_type = [oras.defaults.default_manifest_media_type]
        headers = {"Accept": ";".join(allowed_media_type)}

        get_manifest = f"{self.prefix}://{container.manifest_url()}"  # type: ignore
        response = self.do_request(get_manifest, "GET", headers=headers)
        self._check_200_response(response)
        manifest = response.json()
        return manifest


def download_rafs_based_image():
    """
    Method to download RAFS based vdb files from a oci registry
    """
    rafs_image_downloaded, paths_list = False, None
    nydus_image_command = shutil.which("nydus-image", mode=os.X_OK)
    if nydus_image_command is not None:
        LOG.info(
            "About to download the vulnerability database from %s. This might take a while ...",
            vdb_rafs_database_url,
        )

        try:
            oras_client = oras.client.OrasClient(
                registry=VdbDistributionRegistry()
            )
            rafs_data_dir = tempfile.TemporaryDirectory()
            paths_list = oras_client.pull(
                target=vdb_rafs_database_url,
                outdir=rafs_data_dir.name,
                allowed_media_type=[],
                overwrite=True,
            )

            if (
                paths_list
                and os.path.exists(
                    os.path.join(rafs_data_dir.name, "data.rafs")
                )
                and os.path.exists(
                    os.path.join(rafs_data_dir.name, "meta.rafs")
                )
            ):
                nydus_download_command = [
                    f"{nydus_image_command}",
                    "unpack",
                    "--blob",
                    os.path.join(rafs_data_dir.name, "data.rafs"),
                    "--output",
                    os.path.join(data_dir, "vdb.tar"),
                    "--bootstrap",
                    os.path.join(rafs_data_dir.name, "meta.rafs"),
                ]
                _ = subprocess.run(
                    nydus_download_command,
                    check=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
                if os.path.exists(os.path.join(data_dir, "vdb.tar")):
                    rafs_image_downloaded = True
                    with tarfile.open(
                        os.path.join(data_dir, "vdb.tar"), "r"
                    ) as tarf:
                        tarf.extractall(path=data_dir)
                    os.remove(os.path.join(data_dir, "vdb.tar"))
                else:
                    raise FileNotFoundError("vdb.tar not found")
            else:
                raise FileNotFoundError("data.rafs or meta.rafs not found")

        except Exception:
            LOG.info(
                "Unable to pull the vulnerability database (rafs image) from %s. Trying to pull the non-rafs-based VDB image.",
                vdb_rafs_database_url,
            )
            rafs_image_downloaded = False

    return rafs_image_downloaded, data_dir


def download_image():
    """
    Method to download vdb files from a oci registry
    """
    rafs_image_downloaded, paths_list = download_rafs_based_image()
    if rafs_image_downloaded:
        return paths_list
    LOG.info(
        "About to download the vulnerability database from %s. This might take a while ...",
        vdb_database_url,
    )
    oras_client = oras.client.OrasClient(registry=VdbDistributionRegistry())
    paths_list = oras_client.pull(
        target=vdb_database_url,
        outdir=data_dir,
        allowed_media_type=[],
        overwrite=True,
    )
    for apath in paths_list:
        if apath.endswith(".tar.gz"):
            with tarfile.open(apath, "r") as tarf:
                tarf.extractall(path=data_dir)
            try:
                os.remove(apath)
            except OSError:
                pass
    return paths_list



import json

import yaml

from depscan.lib.utils import find_files


def build_license_data(license_dir, spdx_license_list):
    """Build license data based on the txt files"""
    licenses_dict = {}
    with open(spdx_license_list, encoding="utf-8") as fp:
        spdx_license_data = json.load(fp)
        for slic in spdx_license_data.get("licenses"):
            licenses_dict[slic["licenseId"]] = {
                "title": slic["name"],
                "spdx-id": slic["licenseId"],
                "osi_approved": slic.get("isOsiApproved"),
                "fsf_libre": slic.get("isFsfLibre"),
                "conditions": [f"See {slic['detailsUrl']}"],
                "condition_flag": not slic.get("isOsiApproved"),
            }
    license_files = find_files(license_dir, "txt")
    for lfile in license_files:
        with open(lfile, encoding="utf-8") as fp:
            raw_data = fp.read().split("---")[1]
            ldata = yaml.safe_load(raw_data)
            ldata["condition_flag"] = False
            for cond in ldata["conditions"]:
                if cond in [
                    "document-changes",
                    "network-use-disclose",
                    "disclose-source",
                    "same-license",
                    "same-license--file",
                    "same-license--library",
                ]:
                    ldata["condition_flag"] = True

            licenses_dict[ldata.get("spdx-id").strip().upper()] = ldata
    return licenses_dict


def bulk_lookup(license_dict, pkg_list):
    """Lookup package licenses"""
    pkg_licenses = {}
    for pkg in pkg_list:
        # Failsafe in case the bom file contains incorrect entries
        if not pkg.get("name") or not pkg.get("version"):
            continue
        pkg_key = pkg["name"] + "@" + pkg["version"]
        if pkg.get("vendor"):
            pkg_key = pkg.get("vendor") + ":" + pkg["name"] + "@" + pkg["version"]
        for lic in pkg.get("licenses"):
            if lic == "X11":
                lic = "MIT"
            elif "MIT" in lic:
                lic = "MIT"
            curr_list = pkg_licenses.get(pkg_key, [])
            match_lic = license_dict.get(lic)
            if match_lic:
                curr_list.append(match_lic)
            pkg_licenses[pkg_key] = curr_list
    return pkg_licenses



import os

import pytest

from depscan.lib import analysis
from depscan.lib.bom import get_pkg_list
from depscan.lib.license import build_license_data, bulk_lookup


@pytest.fixture
def test_license_data():
    licenses_dir = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        "..",
        "vendor",
        "choosealicense.com",
        "_licenses",
    )
    spdx_license_list = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        "..",
        "vendor",
        "spdx",
        "json",
        "licenses.json",
    )
    return build_license_data(licenses_dir, spdx_license_list)


def test_lookup(test_license_data):
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom.xml"
    )
    pkg_list = get_pkg_list(test_bom)
    pkg_lic_dict = bulk_lookup(test_license_data, pkg_list)
    assert pkg_lic_dict

    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-dotnet.xml"
    )
    pkg_list = get_pkg_list(test_bom)
    pkg_lic_dict = bulk_lookup(test_license_data, pkg_list)
    assert pkg_lic_dict
    violations_list = []
    for pkg, ll in pkg_lic_dict.items():
        for lic in ll:
            if lic["condition_flag"]:
                violations_list.append(lic)
    assert len(violations_list) == 1


def test_dual_license(test_license_data):
    pkg_lic_dict = bulk_lookup(
        test_license_data,
        [
            {
                "vendor": "npm",
                "name": "jszip",
                "version": "3.2.2",
                "licenses": ["(MIT OR GPL-3.0)"],
            }
        ],
    )
    assert pkg_lic_dict == {
        "npm:jszip@3.2.2": [
            {
                "title": "MIT License",
                "spdx-id": "MIT",
                "featured": True,
                "hidden": False,
                "description": "A short and simple permissive license with conditions only requiring preservation of copyright and license notices. Licensed works, modifications, and larger works may be distributed under different terms and without source code.",
                "how": "Create a text file (typically named LICENSE or LICENSE.txt) in the root of your source code and copy the text of the license into the file. Replace [year] with the current year and [fullname] with the name (or names) of the copyright holders.",
                "using": {
                    "Babel": "https://github.com/babel/babel/blob/master/LICENSE",
                    ".NET": "https://github.com/dotnet/runtime/blob/main/LICENSE.TXT",
                    "Rails": "https://github.com/rails/rails/blob/master/MIT-LICENSE",
                },
                "permissions": [
                    "commercial-use",
                    "modifications",
                    "distribution",
                    "private-use",
                ],
                "conditions": ["include-copyright"],
                "limitations": ["liability", "warranty"],
                "condition_flag": False,
            }
        ]
    }
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-node2.xml"
    )
    pkg_list = get_pkg_list(test_bom)
    pkg_lic_dict = bulk_lookup(test_license_data, pkg_list)
    assert pkg_lic_dict


def test_large_lookup(test_license_data):
    test_bom = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "bom-docker.json"
    )
    pkg_list = get_pkg_list(test_bom)
    pkg_lic_dict = bulk_lookup(test_license_data, pkg_list)
    assert pkg_lic_dict
    analysis.analyse_licenses("nodejs", pkg_lic_dict)


def test_unvendor_license(test_license_data):
    pkg_lic_dict = bulk_lookup(
        test_license_data,
        [
            {
                "vendor": "npm",
                "name": "jszip",
                "version": "3.2.2",
                "licenses": ["CC-BY-NC-1.0"],
            }
        ],
    )
    assert pkg_lic_dict == {
        "npm:jszip@3.2.2": [
            {
                "title": "Creative Commons Attribution Non Commercial 1.0 Generic",
                "spdx-id": "CC-BY-NC-1.0",
                "osi_approved": False,
                "fsf_libre": False,
                "conditions": ["See https://spdx.org/licenses/CC-BY-NC-1.0.json"],
                "condition_flag": True,
            }
        ]
    }
